@@ -19,6 +19,7 @@
  */
 
 #include "landevicelink.h"
+#include "core_debug.h"
 
 #include <sys/socket.h>
 #include <netinet/in.h>
@@ -30,16 +31,24 @@
 #include "downloadjob.h"
 #include "socketlinereader.h"
 
-LanDeviceLink::LanDeviceLink(const QString& d, LinkProvider* a, QTcpSocket* socket)
-    : DeviceLink(d, a)
-    , mSocketLineReader(new SocketLineReader(socket, a))
+LanDeviceLink::LanDeviceLink(const QString& deviceId, LinkProvider* parent, QTcpSocket* socket)
+    : DeviceLink(deviceId, parent)
+    , mSocketLineReader(new SocketLineReader(socket))
 {
-    connect(mSocketLineReader, SIGNAL(disconnected()),
-            this, SLOT(deleteLater()));
     connect(mSocketLineReader, SIGNAL(readyRead()),
             this, SLOT(dataReceived()));
+
+    //We take ownership of the socket.
+    //When the link provider distroys us,
+    //the socket (and the reader) will be
+    //destroyed as well
+    connect(socket, SIGNAL(disconnected()),
+            this, SLOT(deleteLater()));
+    mSocketLineReader->setParent(this);
+    socket->setParent(this);
 }
 
+
 bool LanDeviceLink::sendPackageEncrypted(QCA::PublicKey& key, NetworkPackage& np)
 {
     if (np.hasPayload()) {
@@ -54,7 +63,7 @@ bool LanDeviceLink::sendPackageEncrypted(QCA::PublicKey& key, NetworkPackage& np
 
     //TODO: Actually detect if a package is received or not, now we keep TCP
     //"ESTABLISHED" connections that look legit (return true when we use them),
-    //but that are actually broken
+    //but that are actually broken (until keepalive detects that they are down).
     return (written != -1);
 }
 
@@ -67,9 +76,6 @@ bool LanDeviceLink::sendPackage(NetworkPackage& np)
     }
 
     int written = mSocketLineReader->write(np.serialize());
-    //TODO: Actually detect if a package is received or not, now we keep TCP
-    //"ESTABLISHED" connections that look legit (return true when we use them),
-    //but that are actually broken
     return (written != -1);
 }
 
@@ -80,7 +86,7 @@ void LanDeviceLink::dataReceived()
 
     const QByteArray package = mSocketLineReader->readLine();
 
-    //kDebug(kdeconnect_kded()) << "LanDeviceLink dataReceived" << package;
+    //qCDebug(KDECONNECT_CORE) << "LanDeviceLink dataReceived" << package;
 
     NetworkPackage unserialized(QString::null);
     NetworkPackage::unserialize(package, &unserialized);
@@ -90,7 +96,7 @@ void LanDeviceLink::dataReceived()
         unserialized.decrypt(mPrivateKey, &decrypted);
 
         if (decrypted.hasPayloadTransferInfo()) {
-            kDebug(kdeconnect_kded()) << "HasPayloadTransferInfo";
+            qCDebug(KDECONNECT_CORE) << "HasPayloadTransferInfo";
             DownloadJob* job = new DownloadJob(mSocketLineReader->peerAddress(), decrypted.payloadTransferInfo());
             job->start();
             decrypted.setPayload(job->getPayload(), decrypted.payloadSize());
