@@ -20,36 +20,49 @@
 
 #include "kcm.h"
 
-#include <QtGui/QLabel>
-#include <QtGui/QMenu>
-#include <QtGui/QMenuBar>
-#include <QtGui/QAction>
-#include <QtGui/QStackedLayout>
-#include <QtGui/QListView>
+#include <QLabel>
+#include <QMenu>
+#include <QMenuBar>
+#include <QAction>
+#include <QStackedLayout>
+#include <QListView>
 #include <QDBusConnection>
 #include <QDBusInterface>
 
 #include <KServiceTypeTrader>
 #include <KPluginInfo>
+#include <KPluginMetaData>
 #include <KPluginFactory>
-#include <KStandardDirs>
+#include <KAboutData>
+#include <KLocalizedString>
 
 #include "ui_kcm.h"
 #include "interfaces/dbusinterfaces.h"
 #include "interfaces/devicesmodel.h"
 #include "devicessortproxymodel.h"
-#include "kdebugnamespace.h"
+#include "kdeconnect-version.h"
 
 K_PLUGIN_FACTORY(KdeConnectKcmFactory, registerPlugin<KdeConnectKcm>();)
-K_EXPORT_PLUGIN(KdeConnectKcmFactory("kdeconnect-kcm", "kdeconnect-kcm"))
 
 KdeConnectKcm::KdeConnectKcm(QWidget *parent, const QVariantList&)
-    : KCModule(KdeConnectKcmFactory::componentData(), parent)
+    : KCModule(KAboutData::pluginData("kdeconnect-kcm"), parent)
     , kcmUi(new Ui::KdeConnectKcmUi())
+    , daemon(new DaemonDbusInterface(this))
     , devicesModel(new DevicesModel(this))
-    , currentDevice(0)
-    //, config(KSharedConfig::openConfig("kdeconnectrc"))
+    , currentDevice(nullptr)
 {
+    KAboutData *about = new KAboutData("kdeconnect-kcm",
+                                       i18n("KDE Connect Settings"),
+                                       QLatin1String(KDECONNECT_VERSION_STRING),
+                                       i18n("KDE Connect Settings module"),
+                                       KAboutLicense::KAboutLicense::GPL_V2,
+                                       i18n("(C) 2015 Albert Vaca Cintora"),
+                                       QString(),
+                                       QLatin1String("https://community.kde.org/KDEConnect")
+    );
+    about->addAuthor(i18n("Albert Vaca Cintora"));
+    setAboutData(about);
+
     kcmUi->setupUi(this);
 
     kcmUi->deviceList->setIconSize(QSize(32,32));
@@ -62,55 +75,105 @@ KdeConnectKcm::KdeConnectKcm(QWidget *parent, const QVariantList&)
     kcmUi->progressBar->setVisible(false);
     kcmUi->messages->setVisible(false);
 
+    //Workaround: If we set this directly (or if we set it in the .ui file), the layout breaks
+    kcmUi->noDeviceLinks->setWordWrap(false);
+    QTimer::singleShot(0, [this] { kcmUi->noDeviceLinks->setWordWrap(true); });
+
+    kcmUi->rename_label->setText(daemon->announcedName());
+    kcmUi->rename_edit->setText(daemon->announcedName());
+    setRenameMode(false);
+
     setButtons(KCModule::NoAdditionalButton);
 
     connect(devicesModel, SIGNAL(dataChanged(QModelIndex,QModelIndex)),
             this, SLOT(resetSelection()));
-    connect(kcmUi->deviceList, SIGNAL(pressed(QModelIndex)),
+    connect(kcmUi->deviceList->selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
             this, SLOT(deviceSelected(QModelIndex)));
-    connect(kcmUi->pair_button, SIGNAL(pressed()),
+    connect(kcmUi->pair_button, SIGNAL(clicked()),
             this, SLOT(requestPair()));
-    connect(kcmUi->unpair_button, SIGNAL(pressed()),
+    connect(kcmUi->unpair_button, SIGNAL(clicked()),
             this, SLOT(unpair()));
-    connect(kcmUi->ping_button, SIGNAL(pressed()),
+    connect(kcmUi->ping_button, SIGNAL(clicked()),
             this, SLOT(sendPing()));
+    connect(kcmUi->refresh_button,SIGNAL(clicked()),
+            this, SLOT(refresh()));
+    connect(kcmUi->rename_edit,SIGNAL(returnPressed()),
+            this, SLOT(renameDone()));
+    connect(kcmUi->renameDone_button,SIGNAL(clicked()),
+            this, SLOT(renameDone()));
+    connect(kcmUi->renameShow_button,SIGNAL(clicked()),
+            this, SLOT(renameShow()));
 
 }
 
+void KdeConnectKcm::renameShow()
+{
+    setRenameMode(true);
+}
+
+void KdeConnectKcm::renameDone()
+{
+    QString newName = kcmUi->rename_edit->text();
+    if (newName.isEmpty()) {
+        //Rollback changes
+        kcmUi->rename_edit->setText(kcmUi->rename_label->text());
+    } else {
+        kcmUi->rename_label->setText(newName);
+        daemon->setAnnouncedName(newName);
+    }
+    setRenameMode(false);
+}
+
+void KdeConnectKcm::setRenameMode(bool b) {
+    kcmUi->renameDone_button->setVisible(b);
+    kcmUi->rename_edit->setVisible(b);
+    kcmUi->renameShow_button->setVisible(!b);
+    kcmUi->rename_label->setVisible(!b);
+}
+
 KdeConnectKcm::~KdeConnectKcm()
 {
+    delete kcmUi;
+}
 
+void KdeConnectKcm::refresh()
+{
+    daemon->forceOnNetworkChange();
 }
 
 void KdeConnectKcm::resetSelection()
 {
+    if (!currentDevice) {
+        return;
+    }
     kcmUi->deviceList->selectionModel()->setCurrentIndex(sortProxyModel->mapFromSource(currentIndex), QItemSelectionModel::ClearAndSelect);
 }
 
 void KdeConnectKcm::deviceSelected(const QModelIndex& current)
 {
 
+    kcmUi->noDevicePlaceholder->setVisible(false);
+
     if (currentDevice) {
-        disconnect(currentDevice,SIGNAL(pairingSuccesful()),
-            this, SLOT(pairingSuccesful()));
+        disconnect(currentDevice,SIGNAL(pairingChanged(bool)),
+            this, SLOT(pairingChanged(bool)));
         disconnect(currentDevice,SIGNAL(pairingFailed(QString)),
             this, SLOT(pairingFailed(QString)));
-        disconnect(currentDevice,SIGNAL(unpaired()),
-            this, SLOT(unpaired()));
     }
 
     //Store previous device config
     pluginsConfigChanged();
 
     if (!current.isValid()) {
+        currentDevice = nullptr;
         kcmUi->deviceInfo->setVisible(false);
         return;
     }
 
     currentIndex = sortProxyModel->mapToSource(current);
-    currentDevice = devicesModel->getDevice(currentIndex);
+    currentDevice = devicesModel->getDevice(currentIndex.row());
 
-    bool valid = (currentDevice != NULL && currentDevice->isValid());
+    bool valid = (currentDevice != nullptr && currentDevice->isValid());
     kcmUi->deviceInfo->setVisible(valid);
     if (!valid) {
         return;
@@ -135,27 +198,24 @@ void KdeConnectKcm::deviceSelected(const QModelIndex& current)
         }
     }
 
-    //FIXME: KPluginSelector has no way to remove a list of plugins and load another, so we need to destroy and recreate it each time
+    //KPluginSelector has no way to remove a list of plugins and load another, so we need to destroy and recreate it each time
     delete kcmUi->pluginSelector;
     kcmUi->pluginSelector = new KPluginSelector(this);
-    kcmUi->verticalLayout_2->addWidget(kcmUi->pluginSelector);
+    kcmUi->deviceInfo_layout->addWidget(kcmUi->pluginSelector);
+
+    kcmUi->pluginSelector->setConfigurationArguments(QStringList(currentDevice->id()));
 
     kcmUi->name_label->setText(currentDevice->name());
     kcmUi->status_label->setText(currentDevice->isPaired()? i18n("(paired)") : i18n("(unpaired)"));
 
-    connect(currentDevice,SIGNAL(pairingSuccesful()),
-            this, SLOT(pairingSuccesful()));
+    connect(currentDevice,SIGNAL(pairingChanged(bool)),
+            this, SLOT(pairingChanged(bool)));
     connect(currentDevice,SIGNAL(pairingFailed(QString)),
             this, SLOT(pairingFailed(QString)));
-    connect(currentDevice,SIGNAL(unpaired()),
-            this, SLOT(unpaired()));
 
-    KService::List offers = KServiceTypeTrader::self()->query("KdeConnect/Plugin");
-    QList<KPluginInfo> scriptinfos = KPluginInfo::fromServices(offers);
-
-    QString path = KStandardDirs().resourceDirs("config").first()+"kdeconnect/";
-    KSharedConfigPtr deviceConfig = KSharedConfig::openConfig(path + currentDevice->id());
-    kcmUi->pluginSelector->addPlugins(scriptinfos, KPluginSelector::ReadConfigFile, i18n("Plugins"), QString(), deviceConfig);
+    const QList<KPluginInfo> pluginInfo = KPluginInfo::fromMetaData(KPluginLoader::findPlugins("kdeconnect/"));
+    KSharedConfigPtr deviceConfig = KSharedConfig::openConfig(currentDevice->pluginsConfigFile());
+    kcmUi->pluginSelector->addPlugins(pluginInfo, KPluginSelector::ReadConfigFile, i18n("Plugins"), QString(), deviceConfig);
 
     connect(kcmUi->pluginSelector, SIGNAL(changed(bool)),
             this, SLOT(pluginsConfigChanged()));
@@ -186,46 +246,26 @@ void KdeConnectKcm::unpair()
     currentDevice->unpair();
 }
 
-void KdeConnectKcm::unpaired()
-{
-    DeviceDbusInterface* senderDevice = (DeviceDbusInterface*) sender();
-    devicesModel->deviceStatusChanged(senderDevice->id());
-
-    if (senderDevice != currentDevice) return;
-
-    kcmUi->pair_button->setVisible(true);
-    kcmUi->unpair_button->setVisible(false);
-    kcmUi->progressBar->setVisible(false);
-    kcmUi->ping_button->setVisible(false);
-    kcmUi->status_label->setText(i18n("(unpaired)"));
-}
-
 void KdeConnectKcm::pairingFailed(const QString& error)
 {
     if (sender() != currentDevice) return;
 
-    kcmUi->pair_button->setVisible(true);
-    kcmUi->unpair_button->setVisible(false);
-    kcmUi->progressBar->setVisible(false);
-    kcmUi->ping_button->setVisible(false);
-    kcmUi->status_label->setText(i18n("(unpaired)"));
+    pairingChanged(false);
 
     kcmUi->messages->setText(i18n("Error trying to pair: %1",error));
     kcmUi->messages->animatedShow();
 }
 
-void KdeConnectKcm::pairingSuccesful()
+void KdeConnectKcm::pairingChanged(bool paired)
 {
     DeviceDbusInterface* senderDevice = (DeviceDbusInterface*) sender();
-    devicesModel->deviceStatusChanged(senderDevice->id());
-
     if (senderDevice != currentDevice) return;
 
-    kcmUi->pair_button->setVisible(false);
-    kcmUi->unpair_button->setVisible(true);
-    kcmUi->progressBar->setVisible(false);
-    kcmUi->ping_button->setVisible(true);
-    kcmUi->status_label->setText(i18n("(paired)"));
+    kcmUi->pair_button->setVisible(!paired);
+    kcmUi->unpair_button->setVisible(paired);
+    kcmUi->progressBar->setVisible(senderDevice->pairRequested());
+    kcmUi->ping_button->setVisible(paired);
+    kcmUi->status_label->setText(paired ? i18n("(paired)") : i18n("(unpaired)"));
 }
 
 void KdeConnectKcm::pluginsConfigChanged()
@@ -234,7 +274,7 @@ void KdeConnectKcm::pluginsConfigChanged()
     if (!currentDevice) return;
 
     DeviceDbusInterface* auxCurrentDevice = currentDevice;
-    currentDevice = 0; //HACK to avoid infinite recursion (for some reason calling save on pluginselector emits changed)
+    currentDevice = nullptr; //HACK to avoid infinite recursion (for some reason calling save on pluginselector emits changed)
     kcmUi->pluginSelector->save();
     currentDevice = auxCurrentDevice;
 
@@ -250,6 +290,18 @@ void KdeConnectKcm::save()
 void KdeConnectKcm::sendPing()
 {
     if (!currentDevice) return;
-    QDBusMessage msg = QDBusMessage::createMethodCall("org.kde.kdeconnect", "/modules/kdeconnect/devices/"+currentDevice->id()+"/ping", "org.kde.kdeconnect.device.ping", "sendPing");
-    QDBusConnection::sessionBus().call(msg);
+    currentDevice->pluginCall("ping", "sendPing");
 }
+
+QSize KdeConnectKcm::sizeHint() const
+{
+    return QSize(890,550); //Golden ratio :D
+}
+
+QSize KdeConnectKcm::minimumSizeHint() const
+{
+    return QSize(500,300);
+}
+
+#include "kcm.moc"
+#include "moc_kcm.cpp"
